@@ -15,8 +15,8 @@ use human_panic::setup_panic;
 #[cfg(feature = "password-storage")]
 use keyring::{Keyring, KeyringError};
 use maturin::{
-    develop, source_distribution, write_dist_info, BridgeModel, BuildOptions, CargoToml,
-    Metadata21, PathWriter, PlatformTag, PyProjectToml, PythonInterpreter, Target,
+    develop, new_project, source_distribution, write_dist_info, BridgeModel, BuildOptions,
+    CargoToml, Metadata21, PathWriter, PlatformTag, PyProjectToml, PythonInterpreter, Target,
 };
 use std::env;
 use std::io;
@@ -270,6 +270,21 @@ enum Opt {
         #[structopt(short, long, parse(from_os_str))]
         out: Option<PathBuf>,
     },
+    /// Create a new cargo project
+    #[structopt(name = "new")]
+    NewProject {
+        /// Project path
+        path: String,
+        /// Set the resulting package name, defaults to the directory name
+        #[structopt(long)]
+        name: Option<String>,
+        /// Use mixed Rust/Python project layout
+        #[structopt(long)]
+        mixed: bool,
+        /// Which kind of bindings to use
+        #[structopt(short, long, possible_values = &["pyo3", "rust-cpython", "cffi", "bin"])]
+        bindings: Option<String>,
+    },
     /// Uploads python packages to pypi
     ///
     /// It is mostly similar to `twine upload`, but can only upload python wheels
@@ -364,18 +379,18 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             // Since afaik all other PEP 517 backends also return linux tagged wheels, we do so too
             let tags = match context.bridge {
                 BridgeModel::Bindings(_) => {
-                    vec![context.interpreter[0].get_tag(PlatformTag::Linux, context.universal2)]
+                    vec![context.interpreter[0].get_tag(PlatformTag::Linux, context.universal2)?]
                 }
                 BridgeModel::BindingsAbi3(major, minor) => {
                     let platform = context
                         .target
-                        .get_platform_tag(PlatformTag::Linux, context.universal2);
+                        .get_platform_tag(PlatformTag::Linux, context.universal2)?;
                     vec![format!("cp{}{}-abi3-{}", major, minor, platform)]
                 }
                 BridgeModel::Bin | BridgeModel::Cffi => {
                     context
                         .target
-                        .get_universal_tags(PlatformTag::Linux, context.universal2)
+                        .get_universal_tags(PlatformTag::Linux, context.universal2)?
                         .1
                 }
             };
@@ -621,6 +636,12 @@ fn run() -> Result<()> {
             .context("Failed to build source distribution")?;
         }
         Opt::Pep517(subcommand) => pep517(subcommand)?,
+        Opt::NewProject {
+            path,
+            name,
+            mixed,
+            bindings,
+        } => new_project(path, name, mixed, bindings)?,
         #[cfg(feature = "upload")]
         Opt::Upload { publish, files } => {
             if files.is_empty() {
